#!/usr/bin/env python


"""
This module exports two functions useful to the file integrity checker
'Spot The Difference":
  - parse_options: parses the command line options
  - cfg_parse:     parses the configuration file
"""


import os
from glob import glob
from optparse import OptionParser
from sys import exit


def parse_options():
    """Parse the command line options and return them"""
    usage = "%prog [-h] [-v|-q] [-C config_file] [-c|-u] [-o output_file] [-e]"
    version = "Spot The Difference 0.2.1 - File Integrity Checker"
    parser = OptionParser(usage=usage, version=version)

    parser.add_option("-C", "--configfile", dest="cfg_file", metavar="<FILE>",
                      help="Specify the configuration file",
                      default="/etc/stdiff.conf")
    parser.add_option("-c", "--check", dest="check", action="store_true",
                      help="Check the current state of the filesystem")
    parser.add_option("-u", "--update", dest="update", action="store_true",
                      help="Create or update the database file")
    parser.add_option("-o", "--outfile", dest="outfile", metavar="<FILE>",
                      help="Specify the filename of the final report",
                      default="stdiff.out")
    parser.add_option("-e", "--email", dest="email", action="store_true",
                      help="Turn on email notification")
    parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                      help="Verbose mode")
    parser.add_option("-q", "--quiet", dest="quiet", action="store_true",
                      help="Quiet mode")

    return parser.parse_args()


def cfg_parse(filename):
    """Parse 'spot The Difference' configuration file

    Parse the rules and create a dictionary of:
    - paths (files or directories) to be recursively checked ("root")
    - paths (files or directories) to ignore ("ignore")
    - paths (files or directories) with special, non cascading checks ("dollar")
    - paths (directories) not to descend ("equal")
    Values corresponding to 'root' and 'dollar' keys are themselves
    dictionaries. They are made up of a path (key) and a string of checks
    (value) to perform on that path.
    Any line in the configuration file that is not a rule (or a comment or a
    syntax error!) is a variable setting. Variables are parsed and grouped in a
    dictionary. These two dictionaries are eventually returned.
    """
    paths = {"root": {},
             "ignore": [],
             "dollar": {},
             "equal": []}

    vars = {}

    try:
        cfg_file = open(filename, "rt")
    except IOError:
        exit("\nCould not open '%s'" % filename)

    for i, line in enumerate(cfg_file):

        # strip off comments
        index = line.find("#")
        if (index != -1):
            line = line[:index]

        # Remove leading and trailing blanks
        line = line.strip()

        try:
            if line == "":
                continue

            elif line.startswith("!"):
                paths["ignore"].extend([p for p in glob(line[1:])])

            elif line.startswith("$"):
                path, checks = line[1:].split()
                for p in glob(path):
                    paths["dollar"][p] = checks

            elif line.startswith("="):
                paths["equal"].extend([p for p in glob(line[1:])])

            elif os.path.isabs(line.split()[0]):
                path, checks = line.split()
                for p in glob(path):
                    paths["root"][p] = checks

            else:
                key, value = line.split("=", 2)
                vars[key.rstrip()] = value.lstrip()

        except ValueError:
            exit("'%s' line %d: Syntax Error" % (filename, i+1))

    cfg_file.close()

    return (paths, vars)

