#!/usr/bin/env python

"""
This module is part of the file integrity checker 'Spot The Difference'.
It subcalasses the generic class 'IntegrityChecker' to make it interact with
dbm files.
Since the generic class is meant to interact with database servers, not files,
dummy db and cursor classes have been created.
Also the query methods have been completely overridden.
"""


from spot import IntegrityChecker
import dbhash
import md5
from sys import exit


################################################################################
# Constant definitions                                                         #
################################################################################

# Tuple of available checks in their single-character representation. It will
# help returning values in the right order.
checks = tuple("5spidlugzamc")

# Dictionary of available checks in both their single-character and
# longer-string form. It will help converting from one form to the other.
headers = {"md5":      "5",   # md5 checksum
           "sha":      "s",   # sha1 checksum
           "st_mode":  "p",   # permissions
           "st_ino":   "i",   # inode number
           "st_dev":   "d",   # device number
           "st_nlink": "l",   # number of links
           "st_uid":   "u",   # user ID
           "st_gid":   "g",   # group ID
           "st_size":  "z",   # size
           "st_atime": "a",   # access time
           "st_mtime": "m",   # modification time
           "st_ctime": "c"}   # change time


################################################################################
# Helper Functions                                                             #
################################################################################

def connect(db_name, mode):
    """Open the dbm file and return a dummy db object"""
    try:
        db = dbhash.open(db_name, mode, 0600)
    except:
        exit("Could not open file '%s'" % db_name)

    return dummy_db(db)


################################################################################
# Class definitions                                                            #
################################################################################

class dummy_db:

    """This class implements a dummy database object, just to make 'dbm' files
    callable like a database.
    """

    def __init__(self, db):
        self.db = db

    def cursor(self):
        """Return a dummy cursor object"""
        return dummy_cursor(self.db)

    def commit(self):
        pass

    def close(self):
        """Close the dbm file"""
        self.db.close()


class dummy_cursor:

    """This class implements a dummy database cursor object"""

    def __init__(self, db):
        self.db = db

    def fetchone(self):
        """Return a list made up of a pathname and the values of all checks
        performed on that path. 'None' values are returned if a check is not
        present. It simulates the result of database query.
        """
        values_dict = {}

        try:
            filename, values_str = self.db.next()
        except:
            # End of database records
            return None

        for key, value in [v.split(':') for v in values_str.split()]:
            values_dict[key] = value

        return [filename] + [values_dict.get(c, None) for c in checks]


class spot(IntegrityChecker):

    """File integrity checker with dbm files access"""

    def __init__(self, options, paths, vars_):
        """Call its parent __init__ method"""
        IntegrityChecker.__init__(self, options, paths, vars_)

    def open_db(self):
        """Open the dbm file

        A dummy_db instance and a 'None' table name are returned.
        As requested by the main class, the db_name attribute is set.
        """
        if self.options.check:
            mode = "r"
        else:
            mode = "n"

        self.db_name = self.vars["db"]

        return (connect(self.db_name, mode), None)

    def update_db(self, values):
        """Insert a new item in the dbm file.

        The argument 'values' is a dictionary of checks/values. The value
        corresponding to the key 'path' is the filename the checks apply to.
        Each key of the dbm dictionary is a pathname; the corresponding value is
        a string of whitespace separated checks. Each check is made up of its
        single-character representation, a colon and its value. E.g.:
        {'/bsd': '5:9c20aa9d806accb666a6b23613fdf724 p:33188 z:5074980'}
        Here, the key is the path '/bsd' and the value is a string made up of
        three checks('5', 'p' and 'z') and their corresponding values
        ('9c20aa9d806accb666a6b23613fdf724', '33188' and '5074980')
        """
        filename = values.pop("path").replace("'", "")
        row = " ".join(["%s:%s" % (headers[k], v) for k, v in values.items()])
        self.cursor.db[filename] = row.replace("'", "")

    def select_all(self):
        pass

    def select_path(self):
        """Return a list of all the files stored in the dbm file"""
        return self.cursor.db.keys()

    def select_count(self):
        """Return the number of items in the dbm file"""
        return len(self.cursor.db)

    def truncate_table(self):
        pass

    def print_footer(self, start_time, end_time):
        """Extend the superclass method adding the fingerprint of the database
        file at the end of the report
        """
        IntegrityChecker.print_footer(self, start_time, end_time)

        f = open(self.db_name, "r")
        m = md5.new(f.read())
        f.close()

        self.outfile.write("\nMD5 fingerprint of the database file '%s' is:\n"
                            % self.db_name)
        self.outfile.write("'%s'\n\n%s\n" % (m.hexdigest(), ("*" * 80)))

